require 'base64'
require 'uri'
require 'excon'
require 'active_support/core_ext/hash/slice'

module ActiveMerchant
  module Billing
    class VeritransGateway < Gateway

      # == Veritrans Error Codes
      # 400 Validation error ( Validation Error, merchant send bad request data example; validation error )
      # => invalid transaction type
      # => invalid credit card format
      #
      # 401 Access Denied	Invalid client key or server key
      # 402 Access Denied	Merchant doesn't have access for this payment type
      # 406 Duplicate Order ID	Order ID has been used before or transaction has been paid.
      # 410 Account Inactive	Merchant account is deactivated or not active for a long period
      # 411 Token error	Token id is missing, invalid, or timed out

      self.supported_cardtypes = [:visa, :master]
      self.display_name = 'Veritrans'

      # POST /v2/charge { payment_type: "vtdirect" }
      # Docs http://docs.veritrans.co.id/sandbox/charge.html
      def charge(data, config)
        make_request(:post, config[:url_api] + "/v2/charge", data, config[:server_key])
      end

      # POST /v2/{id}/cancel
      # Docs http://docs.veritrans.co.id/en/api/methods.html#Cancel
      def cancel(transaction_id, config)
        make_request(:post, config[:url_api] + "/v2/#{URI.escape(transaction_id)}/cancel", {}, config[:server_key])
      end

      # POST /v2/{id}/approve
      # Docs http://docs.veritrans.co.id/en/api/methods.html#Approve
      def approve(payment_id, options = {})
        #request_with_logging(:post, config.api_host + "/v2/#{URI.escape(payment_id)}/approve", options)
      end

      # POST /v2/capture
      # Docs http://docs.veritrans.co.id/en/api/methods.html#Capture
      def capture(transaction_id, gross_amount, config)
        make_request(:post, config[:url_api] + "/v2/capture", { transaction_id: transaction_id, gross_amount: gross_amount }, config[:server_key])
      end

  private

      def basic_auth_header(server_key)
        key = Base64.strict_encode64(server_key + ":")
        "Basic #{key}"
      end

      def make_request(method, url, data, server_key = SERVER_KEY)
        default_options = {}

        # Add authentication and content type
        # Docs http://docs.veritrans.co.id/sandbox/introduction.html
        request_options = {
          :path => URI.parse(url).path,
          :headers => {
            :Authorization => basic_auth_header(server_key),
            :Accept => "application/json",
            :"Content-Type" => "application/json",
            :"User-Agent" => "Spree Veritrans gateway engine"
          }
        }

        if method.to_s.upcase == "GET"
          request_options[:query] = URI.encode_www_form(data)
        else
          request_options[:body] = _json_encode(data)
        end

        connection_options = {
          read_timeout: 40,
          write_timeout: 40,
          connect_timeout: 40
        }.deep_merge(default_options)

        request = Excon.new(url, connection_options)
        response = request.send(method, request_options)

        result = _json_decode(response.body)
        result = result.inject({}){|res,(k,v)| res[k.to_sym] = v; res}
        success = result[:status_code] == "200"
        msg = result[:status_message]

        Response.new(success, msg, result, {
          fraud_review: result[:fraud_status],
          authorization: result[:transaction_id],
          error_code: success ? nil : result[:status_code]
        })

      rescue Excon::Errors::SocketError => error
        msg = "Internal server error, no response from backend. Try again later"
        error_response = Excon::Response.new(
          body: "{'status_code': '500', 'status_message': #{msg} }",
          status: '500'
        )
        result = _json_decode(error_response.body)
        result = result.inject({}){|res,(k,v)| res[k.to_sym] = v; res}
        Response.new(false, msg, result, {
          error_code: "500"
        })
      end

      # Failback for activesupport
      def _json_encode(params)
        if defined?(ActiveSupport) && defined?(ActiveSupport::JSON)
          ActiveSupport::JSON.encode(params)
        else
          require 'json' unless defined?(JSON)
          JSON.generate(params)
        end
      end

      def _json_decode(params)
        if defined?(ActiveSupport) && defined?(ActiveSupport::JSON)
          ActiveSupport::JSON.decode(params)
        else
          require 'json' unless defined?(JSON)
          JSON.parse(params)
        end
      end

    end
  end
end
